#include "icpc_gen_helper.h"

#include <sstream>
#include <optional>

using namespace std;
using namespace icpc_gen_helper;

using ll = long long;
using pll = pair<ll,ll>;
using vll = vector<ll>;

const ll MAXN = 1000000;

int main(int argc, char** argv) {
	Init(argc, argv);

	const ll num_leaves = stoll(getParam("--leaves"));
	assert(num_leaves > 0);
	// ll n = num_leaves * 2 - 1;
	// num_symbols = 2 * n + (n - num_leaves)
	//				= 3 * n - num_leaves
	//				= 6 * num_leaves - 3 - num_leaves
	//				= 5 * num_leaves - 3
	
	auto opDistr = [&] {
			double sk, sk1 = 1;
			if (uniform_number(0,7))
				sk = max(1ll, (geometric_distribution<ll>(0.4))(rng));
			else
				sk = uniform_number(0ll, MAXN);
			if (uniform_number(0,1)) swap(sk, sk1);
			return discrete_distribution<int>{sk, sk1};
		}();

	vector<optional<tuple<bool,ll,ll>>> children;
	vll leaves;
	children.emplace_back(nullopt);
	leaves.emplace_back(0);

	const ll degenerate = getParam("--degenerate") ? stoll(getParam("--degenerate")) : 1;
	ll degeneracy = 0;
	if (degenerate != 1 && getParam("--deg"))
		degeneracy = stoll(getParam("--deg"));

	while (leaves.size() < num_leaves) {
		const ll idx = [&] {
				if (degenerate == 0 && (degeneracy == 0 || uniform_number(0ll, degeneracy)))
						return (ll)(leaves.size()-1);
				else if(degenerate == 2 && (degeneracy == 0 || uniform_number(0ll, degeneracy)))
					return 0ll;
				else
					return uniform_number(0ll, ll(leaves.size())-1);
			}();
		const ll i = leaves[idx];

		// make i an inner node

		// create children
		leaves[idx] = children.size();
		leaves.emplace_back(children.size() + 1);
		for (ll k : {0,1})
			children.emplace_back(nullopt);

		children[i] = make_tuple(opDistr(rng),
								 children.size() - 1,
								 children.size() - 2);
	}

	stringstream out;

	const function<void(ll)> print = [&](ll i) {
		out << '(';
		if (children[i]) {
			const auto& [op, l, r] = *children[i];
			print(l);
			out << (op ? '*' : '+');
			print(r);
		}
		out << ')';
	};
	print(0); // 0 is root

	const string s = out.str();
	assert(s.size() <= MAXN);
	assert(s.size() == 5 * num_leaves - 3);

	cout << s << endl;

	return 0;
}
