#include <iostream>
#include <cassert>
#include <tuple>
#include <vector>
#include <numeric>
#include <functional>
#include <queue>

using namespace std;

using ll = long long;
using pll = pair<ll,ll>;
using vll = vector<ll>;

#define MP make_pair
#define sz(a) (ll((a).size()))
#define all(a) (a).begin(),(a).end()

constexpr ll MAXN = 1000000;

ll cnt = 0;
vector<ll> adj[MAXN];
vector<pll> merges;

pair<string_view,pll> go(string_view s) {
	assert(s[0] == '(');
	if(s[1] == ')') {
		adj[cnt++].emplace_back(cnt++);
		return MP(s.substr(2), pll(cnt-2, cnt-1));
	}

	pll l,r;
	tie(s, l) = go(s.substr(1));
	const char op = s[0];
	tie(s, r) = go(s.substr(1));

	if (op == '+') {
		merges.emplace_back(l.second, r.first);
		return MP(s.substr(1), pll(l.first, r.second));
	} else {
		assert(op == '*');
		merges.emplace_back(l.first, r.first);
		merges.emplace_back(l.second, r.second);
		return MP(s.substr(1), l);
	}

}

vector<pll> rem_adj[MAXN];
pll merge(ll& src, ll& snk) {
	vector<ll> pa(cnt);
	iota(pa.begin(), pa.end(), 0);
	const function<ll(ll)> ufind = [&](ll i) { return i==pa[i] ? i : (pa[i] = ufind(pa[i])); };
	for (auto[x,y] : merges) pa[ufind(x)] = ufind(y);
	ll n = 0;
	vector<ll> idx(cnt);
	for (ll i = 0; i < cnt; i++)
		if (pa[i] == i)
			idx[i] = n++;
	ll m = 0;
	for (ll i = 0; i < cnt; i++) {
		const ll ii = idx[ufind(i)];
		for (ll j : adj[i])
			rem_adj[ii].emplace_back(idx[ufind(j)], m++);
	}

	src = idx[ufind(src)];
	snk = idx[ufind(snk)];
	return MP(n,m);
}

int main() {
	ios::sync_with_stdio(0), cin.tie(0);
	string s; cin >> s;
	auto[src,snk] = go(s).second;
	const auto [n,m] = merge(src, snk);

	vector<ll> node_remaining(n, 0);
	vll top_order;
	const function<void(ll i)> topsort_dfs = [&](ll i) {
		if (node_remaining[i]) return;
		node_remaining[i] = true;
		for (auto[j,e] : rem_adj[i]) topsort_dfs(j);
		top_order.emplace_back(i);
	};
	topsort_dfs(src);
	vector<bool> edge_visited(m, false);
	const auto update_remaining = [&] {
		fill(all(node_remaining), 0);
		bool done = true;
		for (ll i : top_order) {
			for (ll k = 0; k < sz(rem_adj[i]); k++) {
				const auto&[j,e] = rem_adj[i][k];
				const ll v = !edge_visited[e] + node_remaining[j];
				if (v > 0) {
					node_remaining[i] = max(node_remaining[i], v);
					done = false;
				} else {
					rem_adj[i][k] = move(rem_adj[i].back());
					rem_adj[i].pop_back();
					k--;
				}
			}
		}
		return done;
	};

	const function<void(ll)> dfs = [&](ll i) {
		tuple<ll,ll,ll> jj(-1, -1, -1);
		for (auto&[j,e] : rem_adj[i])
			jj = max(jj, make_tuple(node_remaining[j] + !edge_visited[e], e, j));
		if (get<0>(jj) > 0) {
			edge_visited[get<1>(jj)] = true;
			dfs(get<2>(jj));
		}
	};

	ll res = 0;
	do {
		dfs(src);
		res++;
	} while (!update_remaining());

	cout << res << endl;

	return 0;
}
