#include <iostream>
#include <cassert>
#include <tuple>
#include <vector>
#include <numeric>
#include <functional>
#include <queue>

using namespace std;

using ll = long long;
using pll = pair<ll,ll>;

#define MP make_pair
#define sz(a) (ll((a).size()))

constexpr ll MAXN = 1000000;

ll cnt = 0;
vector<ll> adj[MAXN];
vector<pll> merges;

pair<string_view,pll> go(string_view s) {
	if(s[1] == ')') {
		adj[cnt++].emplace_back(cnt++);
		return MP(s.substr(2), pll(cnt-2, cnt-1));
	}

	pll l,r;
	tie(s, l) = go(s.substr(1));
	const char op = s[0];
	tie(s, r) = go(s.substr(1));

	if (op == '+') {
		merges.emplace_back(l.second, r.first);
		return MP(s.substr(1), pll(l.first, r.second));
	} else {
		merges.emplace_back(l.first, r.first);
		merges.emplace_back(l.second, r.second);
		return MP(s.substr(1), l);
	}

}

ll merge(ll& src, ll& snk) {
	vector<ll> pa(cnt);
	iota(pa.begin(), pa.end(), 0);
	const function<ll(ll)> ufind = [&](ll i) { return i==pa[i] ? i : (pa[i] = ufind(pa[i])); };
	for (auto[x,y] : merges) pa[ufind(x)] = ufind(y);
	ll n = 0;
	vector<ll> idx(cnt);
	for (ll i = 0; i < cnt; i++)
		if (pa[i] == i)
			idx[i] = n++;
	vector<ll> nadj[n];
	for (ll i = 0; i < cnt; i++)
		for (ll j : adj[i])
			nadj[idx[ufind(i)]].emplace_back(idx[ufind(j)]);
	for (ll i = 0; i < n; i++)
		adj[i] = move(nadj[i]);

	src = idx[ufind(src)];
	snk = idx[ufind(snk)];
	return n;
}

int main() {
	ios::sync_with_stdio(0), cin.tie(0);
	string s; cin >> s;
	auto[src,snk] = go(s).second;
	const ll n = merge(src, snk);

	//cerr << "n = " << n << endl;
	//for (ll i = 0; i < n; i++) {
	//	cerr << i << " :";
	//	for (ll j : adj[i])
	//		cerr << " " << j;
	//	cerr << endl;
	//}

	vector<ll> num_in(n);
	for (ll i = 0; i < n; i++)
		for (ll j : adj[i])
			num_in[j]++;
	const auto in_deg = num_in;

	queue<ll> q;
	assert(in_deg[src]==0);
	q.emplace(src);
	ll res = 0;
	while (!q.empty()) {
		ll i = q.front(); q.pop();
		if (in_deg[i] > sz(adj[i])) {
			res += in_deg[i];
			continue;
		}

		for (ll j : adj[i])
			if (--num_in[j] == 0)
				q.emplace(j);
	}
	cout << res << endl;
	
	return 0;
}
